import uuid
import unidecode
import re
from django.db import models
from django.contrib.auth.models import AbstractUser
from django.contrib.auth.validators import UnicodeUsernameValidator
from django.utils.translation import gettext_lazy as _
from django.utils import timezone
from MRIOutils.settings import MAX_USERNAME_LENGTH
from ckeditor.fields import RichTextField
from django.db.models import Q

class User(AbstractUser):
    """
    Surcharge user pour ajouter info mri
    """
    USER = "user"
    ADM = "administrator"
    GUEST = "guest"
    ACCOUNT_TYPE_CHOICES = [
        (USER, "user"),
        (ADM, "administrator"),
        (GUEST, "guest")]
    account_type = models.CharField(
        max_length=15,
        choices=ACCOUNT_TYPE_CHOICES,
        default=USER)
    locked = models.BooleanField(
        default=False)

    class Meta:
        ordering = ['account_type', 'username']

    def save(self, *args, **kwargs):
        self.first_name = self.first_name.lower()
        self.last_name = self.last_name.lower()
        self.email = self.email.lower()
        AbstractUser.save(self, *args, **kwargs)

class UserTrash(models.Model):
    """
    Sauvegarde ldif des user effacé
    """
    username = models.CharField(max_length=150)
    delete_date = models.DateTimeField(auto_now_add=True)
    ldif = models.TextField()

class NewUser(models.Model):
    """
    Table des user en attente de validation
    """
    username_validator = UnicodeUsernameValidator()

    username = models.CharField(
        _('username'),
        max_length=150,
        help_text=_('Required. 150 characters or fewer. Letters, digits and @/./+/-/_ only.'),
        validators=[username_validator],
    )
    first_name = models.CharField(_('first name'), max_length=150, blank=True)
    last_name = models.CharField(_('last name'), max_length=150, blank=True)
    email = models.EmailField(_('email address'), blank=True, db_index=True)
    password = models.CharField(_('password'), max_length=200)
    email_confirmation_uuid = models.UUIDField(default=uuid.uuid4,
                                               unique=True,
                                               editable=False)
    email_confirmed = models.BooleanField(_('email confirmed'), default=False)
    inscription_date = models.DateTimeField(auto_now_add=True, db_index=True)

    def generate_username(self):
        regex = re.compile('[^a-zA-Z]')
        first_name_decoded = regex.sub('', unidecode.unidecode(self.first_name))
        last_name_decoded = regex.sub('', unidecode.unidecode(self.last_name))
        val = "{0}{1}".format(first_name_decoded[0],last_name_decoded).lower()
        x=1
        y=1
        while True:
            while len(val) > MAX_USERNAME_LENGTH:
                val = val[:-1]
            if x == 1 and User.objects.filter(username=val).count() == 0 and NewUser.objects.filter(username=val).count() == 0:
                return val
            else:
                if x < len(first_name_decoded):
                    x += 1
                    new_val =  "{0}{1}".format(first_name_decoded[:x],last_name_decoded).lower()
                    while len(new_val) > MAX_USERNAME_LENGTH:
                        new_val = new_val[:-1]
                    if User.objects.filter(username=new_val).count() == 0 and NewUser.objects.filter(username=new_val).count() == 0:
                        return new_val
                elif y < 100:
                    new_val =  "{0}{1}{2}".format(first_name_decoded[:x],last_name_decoded, y).lower()
                    while len(new_val) > MAX_USERNAME_LENGTH:
                        new_val = new_val[:-1]
                    if User.objects.filter(username=new_val).count() == 0 and NewUser.objects.filter(username=new_val).count() == 0:
                        return new_val
                    y += 1
                else:
                    raise Exception("Impossible de générer un username, toute les options ont été épuisé...")

    def save(self, *args, **kwargs):
        if self._state.adding is True:
            self.username = self.generate_username()
        super(NewUser, self).save(*args, **kwargs)


class WaitingValidationMails(models.Model):
    """
    Liste des mails en attente de validation par admin
    """
    user = models.OneToOneField(User, on_delete=models.CASCADE)
    new_email = models.EmailField(_('new email address'), blank=False)
    changed_date = models.DateTimeField(default=timezone.now)
    email_confirmed = models.BooleanField(_('email confirmed'), default=False)
    email_confirmation_uuid = models.UUIDField(default=uuid.uuid4,
                                               unique=True,
                                               editable=False)


class ChangePasswdLink(models.Model):
    """
    Table avec les liens uniques de changement de mdp
    """
    user = models.ForeignKey(User, on_delete=models.CASCADE)
    link_uuid = models.UUIDField(default=uuid.uuid4,
                                 unique=True,
                                 editable=False, db_index=True)
    send_on = models.DateTimeField(default=timezone.now)


class MailsModels(models.Model):
    """
    Table avec texte et titre pour les mails personnalisables
    """
    mail_name = models.CharField(max_length=30)
    title = models.CharField(_('Title'), max_length=100)
    body_fr = RichTextField(_('French text'))
    body_en = RichTextField(_('English text'))
    last_updated = models.DateTimeField(auto_now=True)
    test_for = models.OneToOneField(User, on_delete=models.CASCADE,
                                    blank=True, null=True)

    class Meta:
        constraints = [
            models.UniqueConstraint(fields=['mail_name', 'test_for'], name='unique_mail_name_by_user')
        ]
